@@ -11,8 +11,20 @@
 #
 # See http://www.latex-project.org/lppl.txt for details.
 #
-# Running otfinfo on Linux
 #
 #CODEbegin
-otfinfo -h
+TeXfile=`basename $0 .sh`
+cat << _EndOfFile_ > $TeXfile.tex
+1 .Zeile\par
+\directlua name{\jobname: \the\inputlineno}{
+  a=3
+  b=4
+  c="c"
+  tex.sprint(a^2+b^2 .. "=" .. c^2)}\par
+2. Zeile
+\bye
+_EndOfFile_
+
+luatex --interaction=batchmode $TeXfile
+grep -A4 error $TeXfile.log >$TeXfile.txt
 #CODEend
