@@ -6,11 +6,11 @@
 --       AUTHOR:  Khaled Hosny, Élie Roux, Philipp Gesang
 --      VERSION:  2.5
 --      LICENSE:  GPL v2.0
---     MODIFIED:  2014-05-15 21:47:39+0200
+--     MODIFIED:  2014-08-10 23:13:18+0200
 -----------------------------------------------------------------------
 
 luaotfload          = luaotfload or { }
-local version       = "2.5" --- <int: major>.<int: minor>-<int: fixes>
+local version       = "2.5-4" --- <int: major>.<int: minor>-<int: fixes>
 luaotfload.version  = version
 luaotfload.self     = "luaotfload-tool"
 
@@ -1405,7 +1405,7 @@ local process_cmdline = function ( ) -- unit -> jobspec
         query        = "",
         log_level    = nil,
         bisect       = nil,
-        config       = { db = { }, misc = { }, run = { }, paths = { } },
+        config       = { db = { }, misc = { }, run = { live = false }, paths = { } },
     }
 
     local long_options = {
