@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: tlmgrgui.pl 32836 2014-02-01 03:11:02Z preining $
+# $Id: tlmgrgui.pl 34070 2014-05-16 23:49:53Z karl $
 #
 # Copyright 2009-2014 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -15,8 +15,8 @@
 $^W = 1;
 use strict;
 
-my $guisvnrev = '$Revision: 32836 $';
-my $guidatrev = '$Date: 2014-02-01 04:11:02 +0100 (Sat, 01 Feb 2014) $';
+my $guisvnrev = '$Revision: 34070 $';
+my $guidatrev = '$Date: 2014-05-17 01:49:53 +0200 (Sat, 17 May 2014) $';
 my $tlmgrguirevision;
 if ($guisvnrev =~ m/: ([0-9]+) /) {
   $tlmgrguirevision = $1;
@@ -38,7 +38,6 @@ use File::Glob;
 
 use Pod::Text;
 
-#use TeXLive::Splashscreen;
 #use Devel::Leak;
 
 use TeXLive::TLUtils qw(setup_programs platform_desc win32 debug);
@@ -158,8 +157,8 @@ $init_paper_subs{"xdvi"} = \&init_paper_xdvi;
 $init_paper_subs{"pdftex"} = \&init_paper_pdftex;
 $init_paper_subs{"dvips"} = \&init_paper_dvips;
 $init_paper_subs{"context"} = \&init_paper_context;
-$init_paper_subs{"dvipdfm"} = \&init_paper_dvipdfm;
 $init_paper_subs{"dvipdfmx"} = \&init_paper_dvipdfmx;
+$init_paper_subs{"psutils"} = \&init_paper_psutils;
 
 
 guimain();
@@ -1312,10 +1311,6 @@ sub init_paper_dvips {
   @{$papers{"dvips"}} = TeXLive::TLPaper::get_paper_list("dvips");
   $currentpaper{"dvips"} = ${$papers{"dvips"}}[0];
 }
-sub init_paper_dvipdfm {
-  @{$papers{"dvipdfm"}} = TeXLive::TLPaper::get_paper_list("dvipdfm");
-  $currentpaper{"dvipdfm"} = ${$papers{"dvipdfm"}}[0];
-}
 sub init_paper_context {
   if (defined($localtlpdb->get_package("bin-context"))) {
     @{$papers{"context"}} = TeXLive::TLPaper::get_paper_list("context");
@@ -1326,6 +1321,10 @@ sub init_paper_dvipdfmx {
   @{$papers{"dvipdfmx"}} = TeXLive::TLPaper::get_paper_list("dvipdfmx");
   $currentpaper{"dvipdfmx"} = ${$papers{"dvipdfmx"}}[0];
 }
+sub init_paper_psutils {
+  @{$papers{"psutils"}} = TeXLive::TLPaper::get_paper_list("psutils");
+  $currentpaper{"psutils"} = ${$papers{"psutils"}}[0];
+}
 
 
 sub init_all_papers {
@@ -2303,7 +2302,8 @@ after dismissing this dialogue.
 After the update, the TeX Live manager will terminate.
 You can then restart it to proceed with further updates."); 
     if ($min_action) {
-      $t .= "\n\n" . __("(At least %s further updates are available after tlmgr has been updated.)", $min_action);
+      $t .= "\n\n"
+. __("(Further updates will be available after tlmgr has been updated.)");
     }
     $t .= "\n\n" . __("Please wait a bit after the program has terminated so that the update can be completed.") if win32();
     $sw->add("Label", -text => $t)->pack(-padx => "3m", -pady => "3m");
