@@ -1,9 +1,9 @@
 #!/usr/bin/env perl
-# $Id: updmap.pl 31853 2013-10-07 22:58:25Z karl $
+# $Id: updmap.pl 34884 2014-08-09 12:20:53Z preining $
 # updmap - maintain map files for outline fonts.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2011, 2012, 2013 Norbert Preining
+# Copyright 2011, 2012, 2013, 2014 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
@@ -17,7 +17,6 @@
 #
 # TODO
 # - check all other invocations
-# - after TL2012? Maybe remove support for reading updmap-local.cfg
 
 my $TEXMFROOT;
 
@@ -25,7 +24,7 @@ BEGIN {
   $^W=1;
   $TEXMFROOT = `kpsewhich -var-value=TEXMFROOT`;
   if ($?) {
-    print STDERR "updmap: Cannot find TEXMFROOT, aborting!\n";
+    print_error("Cannot find TEXMFROOT, aborting!\n");
     exit 1;
   }
   chomp($TEXMFROOT);
@@ -33,7 +32,14 @@ BEGIN {
 }
 
 
-my $version = '$Id: updmap.pl 31853 2013-10-07 22:58:25Z karl $';
+my $svnid = '$Id: updmap.pl 34884 2014-08-09 12:20:53Z preining $';
+my $lastchdate = '$Date: 2014-08-09 14:20:53 +0200 (Sat, 09 Aug 2014) $';
+$lastchdate =~ s/^\$Date:\s*//;
+$lastchdate =~ s/ \(.*$//;
+my $svnrev = '$Revision: 34884 $';
+$svnrev =~ s/^\$Revision:\s*//;
+$svnrev =~ s/\s*\$$//;
+my $version = "svn$svnrev ($lastchdate)";
 
 use Getopt::Long qw(:config no_autoabbrev ignore_case_always);
 use strict;
@@ -164,6 +170,25 @@ sub main {
     exit (0);
   }
 
+  #
+  # check if we are in *hidden* sys mode, in which case we switch
+  # to sys mode
+  # Nowdays we use -sys switch instead of simply overriding TEXMFVAR
+  # and TEXMFCONFIG
+  # This is used to warn users when they run updmap in usermode the first time.
+  # But it might happen that this script is called via another wrapper that
+  # sets TEXMFCONFIG and TEXMFVAR, and does not pass on the -sys option.
+  # for this case we check whether the SYS and non-SYS variants agree,
+  # and if, then switch to sys mode (with a warning)
+  if (($TEXMFSYSCONFIG eq $TEXMFCONFIG) && ($TEXMFSYSVAR eq $TEXMFVAR)) {
+    if (!$opts{'sys'}) {
+      print_warning("hidden sys mode found, switching to sys mode.\n");
+      $opts{'sys'} = 1;
+    }
+  }
+
+
+
   if ($opts{'sys'}) {
     # we are running as updmap-sys, make sure that the right tree is used
     $texmfconfig = $TEXMFSYSCONFIG;
@@ -172,12 +197,12 @@ sub main {
 
   if ($opts{'dvipdfmoutputdir'} && !defined($opts{'dvipdfmxoutputdir'})) {
     $opts{'dvipdfmxoutputdir'} = $opts{'dvipdfmoutputdir'};
-    printf STDERR "Using --dvipdfmoutputdir options for dvipdfmx, but please use --dvipdfmxoutputdir\n";
+    print_warning("Using --dvipdfmoutputdir options for dvipdfmx, but please use --dvipdfmxoutputdir\n");
   }
 
   if ($opts{'dvipdfmoutputdir'} && $opts{'dvipdfmxoutputdir'} &&
       $opts{'dvipdfmoutputdir'} ne $opts{'dvipdfmxoutputdir'}) {
-    printf STDERR "Options for --dvipdfmoutputdir and --dvipdfmxoutputdir do not agree\nplease use only --dvipdfmxoutputdir. Exiting.\n";
+    print_error("Options for --dvipdfmoutputdir and --dvipdfmxoutputdir do not agree\nplease use only --dvipdfmxoutputdir. Exiting.\n");
     exit(1);
   }
 
@@ -201,10 +226,10 @@ sub main {
         } elsif ($settings{$o}{'type'} eq "any") {
           print "(any string)\n";
         } else {
-          print "strange: unknown type of option $o\nplease report\n";
+          print_warning("strange: unknown type of option $o\nplease report\n");
         }
       } else {
-        print "$prg: unknown option: $o\n";
+        print_warning("unknown option: $o\n");
       }
     }
     exit 0;
@@ -248,26 +273,22 @@ sub main {
     }
     #
     # search for TEXMFLOCAL/web2c/updmap.cfg
-    # check for compatibility with old updmap-local.cfg
     my @tmlused;
     for my $tml (@TEXMFLOCAL) {
       my $TMLabs = Cwd::abs_path($tml);
       next if (!$TMLabs);
-      my $oldfound = 0;
-      if (-r "$TMLabs/web2c/updmap-local.cfg") {
-        push @tmlused, "$TMLabs/web2c/updmap-local.cfg";
-        warning("Old configuration file\n  $TMLabs/web2c/updmap-local.cfg\nfound! ");
-        $oldfound = 1;
-      }
       if (-r "$TMLabs/web2c/updmap.cfg") {
-        if ($oldfound) {
-          warning("Will read it *instead* of\n  $TMLabs/web2c/updmap.cfg\n");
-        } else {
-          push @tmlused, "$TMLabs/web2c/updmap.cfg";
-        }
+        push @tmlused, "$TMLabs/web2c/updmap.cfg";
+      }
+      #
+      # at least check for old updmap-local.cfg and warn!
+      if (-r "$TMLabs/web2c/updmap-local.cfg") {
+        print_warning("=============================\n");
+        print_warning("Old configuration file\n  $TMLabs/web2c/updmap-local.cfg\n");
+        print_warning("found! This file is *not* evaluated anymore, please move the information\n");
+        print_warning("to the file $TMLabs/updmap.cfg!\n");
+        print_warning("=============================\n");
       }
-      warning("Please consider moving the information from updmap-local.cfg to\n  $TMLabs/web2c/updmap.cfg\n")
-        if ($oldfound);
     }
     #
     # updmap (user):
@@ -334,7 +355,7 @@ sub main {
         $v = "\"$v\"" if ($v =~ m/\s/);
         print "$o=$v ($vo)\n";
       } else {
-        printf STDERR "$prg: unknown option: $o\n";
+        print_warning("unknown option: $o\n");
       }
     }
     exit 0;
@@ -388,7 +409,7 @@ sub main {
   my $cmd;
   if ($opts{'edit'}) {
     if ($opts{"dry-run"}) {
-      printf STDERR "No, are you joking, you want to edit with --dry-run?\n";
+      print_error("No, are you joking, you want to edit with --dry-run?\n");
       exit 1;
     }
     # it's not a good idea to edit updmap.cfg manually these days,
@@ -434,14 +455,14 @@ sub main {
       merge_settings_replace_kanji();
       my @missing = read_map_files();
       if (@missing) {
-        print STDERR "\nERROR:  The following map file(s) couldn't be found:\n"; 
+        print_error("The following map file(s) couldn't be found:\n"); 
         for my $m (@missing) {
           my $orig = $alldata->{'maps'}{$m}{'origin'};
-          print STDERR "\t$m (in $orig)\n";
+          print_error("\t$m (in $orig)\n");
         }
-        print STDERR "\n\tDid you run mktexlsr?\n\n" .
+        print_error("Did you run mktexlsr?\n\n" .
           "\tYou can disable non-existent map entries using the option\n".
-          "\t  --syncwithtrees.\n\n";
+          "\t  --syncwithtrees.\n\n");
         exit 1;
       }
       merge_data();
@@ -486,7 +507,7 @@ sub getFonts {
           if (defined($alldata->{'maps'}{$m}{'fonts'}{$k})) {
             push @lines, "$k " . $alldata->{'maps'}{$m}{'fonts'}{$k};
           } else {
-            print "undefined fonts for $k in $m   ?!?!?\n";
+            print_warning("undefined fonts for $k in $m   ?!?!?\n");
           }
           print LOG "$k\n" unless $opts{'dry-run'};
         }
@@ -852,8 +873,8 @@ sub cidx2dvips {
     # now we have the following format
     #  <word> <word> <word> some options like -e or -s
     if ($_ !~ m/([^ ][^ ]*) ([^ ][^ ]*) ([^ ][^ ]*)( (.*))?$/) {
-      print STDERR "cidx2dvips warning: Cannot translate font line:\n==> $l\n";
-      print STDERR "Current translation status: ==>$_==\n";
+      print_warning("cidx2dvips warning: Cannot translate font line:\n==> $l\n");
+      print_warning("Current translation status: ==>$_==\n");
       next;
     }
     my $tfmname = $1;
@@ -875,7 +896,7 @@ sub cidx2dvips {
       if ($italicmax > 0) {
         # we have already a definition of SlantFont via ,Italic or ,BoldItalic
         # warn the user that larger one is kept
-        print STDERR "cidx2dvips warning: Double slant specified via Italic and -s:\n==> $l\n==> Using only the biggest slant value.\n";
+        print_warning("cidx2dvips warning: Double slant specified via Italic and -s:\n==> $l\n==> Using only the biggest slant value.\n");
       }
       $italicmax = $1 if ($1 > $italicmax);
       $opts =~ s/-s ([.0-9-][.0-9-]*)//;
@@ -1214,8 +1235,8 @@ sub mkMaps {
         print_and_log ("\n");
       }
     } else {
-      print STDERR "$prg: Warning: File $d/$f doesn't exist.\n";
-      print LOG    "Warning: File $d/$f doesn't exist.\n" 
+      print_warning("File $d/$f doesn't exist.\n");
+      print LOG     "Warning: File $d/$f doesn't exist.\n" 
         unless $opts{'dry-run'};
     }
   }
@@ -1490,7 +1511,7 @@ sub disable_map {
   } else {
     # disable a Map type that might be activated in a lower ranked updmap.cfg
     if (!defined($alldata->{'maps'}{$map})) {
-      warning("$prg: map file not present, nothing to disable: $map\n");
+      print_warning("map file not present, nothing to disable: $map\n");
       return;
     }
     my $orig = $alldata->{'maps'}{$map}{'origin'};
@@ -1857,7 +1878,7 @@ sub merge_settings_replace_kanji {
             $alldata->{'updmap'}{$l}{'maps'}{$m}{'line'};
           $alldata->{'updmap'}{$l}{'maps'}{$newm}{'original'} = $m;
         } else {
-          print "$prg: generated map $newm (from $m) does not exists, not activating it!\n";
+          print_warning("generated map $newm (from $m) does not exists, not activating it!\n");
         }
         # in any case delete the @kanji...@ entry line, such a map will
         # never exist
@@ -1882,7 +1903,6 @@ sub merge_settings_replace_kanji {
 
 sub read_updmap_file {
   my $fn = shift;
-  my $is_old_local = ($fn =~ m/updmap-local.cfg/ ? 1 : 0);
   my %data;
   if (!open(FN,"<$fn")) {
     die ("Cannot read $fn: $!");
@@ -1896,15 +1916,6 @@ sub read_updmap_file {
   for (@lines) {
     $i++;
     chomp;
-    if ($is_old_local) {
-      # in case we read an old updmap-local.cfg we have to make sure
-      # that the disable lines
-      #       ^#!foo.map
-      # are rewritten to proper disable lines
-      #       ^#! Map foo.map
-      # we are guessing here the type of the map (namely Map)
-      $_ =~ s/^#!([^ ])/#! Map $1/;
-    }
     next if /^\s*$/;
     next if /^\s*#$/;
     next if /^\s*#[^!]/;
@@ -1919,10 +1930,10 @@ sub read_updmap_file {
       if ($b eq "Map" || $b eq "MixedMap" || $b eq "KanjiMap") {
         my $c = shift @rest;
         if (!defined($c)) {
-          warning("$prg: apparently not a real disable line, ignored: $_\n");
+          print_warning("apparently not a real disable line, ignored: $_\n");
         } else {
           if (defined($data{'maps'}{$c})) {
-            warning("$prg: double mention of $c in $fn\n");
+            print_warning("double mention of $c in $fn\n");
           }
           $data{'maps'}{$c}{'status'} = 'disabled';
           $data{'maps'}{$c}{'type'} = $b;
@@ -1932,25 +1943,25 @@ sub read_updmap_file {
       next;
     }
     if (@rest) {
-      warning("$prg: line $i in $fn contains a syntax error, more than two words!\n");
+      print_warning("line $i in $fn contains a syntax error, more than two words!\n");
     }
     if (defined($settings{$a})) {
       if (check_option($a, $b)) {
         $data{'setting'}{$a}{'val'} = $b;
         $data{'setting'}{$a}{'line'} = $i;
       } else {
-        warning("$prg: unknown setting for $a: $b, ignored!\n");
+        print_warning("unknown setting for $a: $b, ignored!\n");
       }
     } elsif ($a eq "Map" || $a eq "MixedMap" || $a eq "KanjiMap") {
       if (defined($data{'maps'}{$b}) && $data{'maps'}{$b}{'type'} ne $a) {
-        warning("$prg: double mention of $b with conflicting types in $fn\n");
+        print_warning("double mention of $b with conflicting types in $fn\n");
       } else {
         $data{'maps'}{$b}{'type'} = $a;
         $data{'maps'}{$b}{'status'} = 'enabled';
         $data{'maps'}{$b}{'line'} = $i;
       }
     } else {
-      warning("$prg: unrecognized line $i in $fn: $_\n");
+      print_warning("unrecognized line $i in $fn: $_\n");
     }
   }
   return \%data;
@@ -2034,11 +2045,11 @@ sub read_map_files {
               } else {
                 $maporig = "from " . $alldata->{'maps'}{$fontorig}{'origin'};
               }
-              warning("$prg: font $font is defined multiple times:\n");
-              warning("$prg:   $fontorig ($maporig)\n");
-              warning("$prg:   $m (from $f) (used)\n");
+              print_warning("font $font is defined multiple times:\n");
+              print_warning("  $fontorig ($maporig)\n");
+              print_warning("  $m (from $f) (used)\n");
             } else {
-              warning("$prg: font $font is multiply defined in $m, using an arbitrary instance!\n");
+              print_warning("font $font is multiply defined in $m, using an arbitrary instance!\n");
             }
           }
           $alldata->{'fonts'}{$font}{'origin'} = $m;
@@ -2126,28 +2137,31 @@ sub reset_root_home {
       my (undef,undef,undef,undef,undef,undef,undef,$roothome) = getpwuid(0);
       if (defined($roothome)) {
         if ($envhome ne $roothome) {
-          warning("$prg: resetting \$HOME value (was $envhome) to root's "
+          print_warning("resetting \$HOME value (was $envhome) to root's "
             . "actual home ($roothome).\n");
           $ENV{'HOME'} = $roothome;
         } else {
           # envhome and roothome do agree, nothing to do, that is the good case
         }
       } else { 
-        warning("$prg: home of root not defined, strange!\n");
+        print_warning("home of root not defined, strange!\n");
       }
     }
   }
 }
 
-sub warning {
-  print STDERR @_;
+sub print_warning {
+  print STDERR "$prg [WARNING]: ", @_ if (!$opts{'quiet'}) 
+}
+sub print_error {
+  print STDERR "$prg [ERROR]: ", @_;
 }
 
 
 # help, version.
 
 sub version {
-  my $ret = sprintf "%s (TeX Live, multi) version %s\n", $prg, $version;
+  my $ret = sprintf "%s version %s\n", $prg, $version;
   return $ret;
 }
 
@@ -2278,21 +2292,15 @@ Explanation of trees and files normally used:
   
   (where YYYY is the TeX Live release version).
   
-  There is one exception to keep upgradability from earlier versions
-  of TeX Live: if a file TEXMFLOCAL/web2c/updmap-local.cfg exists
-  (formerly used by tlmgr to merge local fonts), then the file
-  TEXMFLOCAL/web2c/updmap.cfg is ignored (if it exists) and that
-  updmap-local.cfg is used instead.  In this case, updmap recognizes the
-  previous syntax for disabling map files in updmap-local.cfg (this
-  syntax is different from what is used now).
-
   According to the actions, updmap might write to one of the given files
   or create a new updmap.cfg, described further below.
 
-  Where changes are saved: if config files are given on the command
-  line, then the first one given will be used to save any changes from
-  --setoption, --enable or --disable.  If the config files are taken
-  from kpsewhich output, then the algorithm is more complex:
+Where changes are saved: 
+
+  If config files are given on the command line, then the first one 
+  given will be used to save any changes from --setoption, --enable 
+  or --disable.  If the config files are taken from kpsewhich output, 
+  then the algorithm is more complex:
 
     1) If \$TEXMFCONFIG/web2c/updmap.cfg or \$TEXMFHOME/web2c/updmap.cfg
     appears in the list of used files, then the one listed first by
@@ -2307,45 +2315,47 @@ Explanation of trees and files normally used:
   can be overridden for system-wide using TEXMFLOCAL, and then system
   settings can be overridden again for a particular using using TEXMFHOME.
 
-  Resolving multiple definitions of a font:
-    If a font is defined in more than one map file, then the definition
-    coming from the first-listed updmap.cfg is used.  If a font is
-    defined multiple times within the same map file, one is chosen
-    arbitrarily.  In both cases a warning is issued.
+Resolving multiple definitions of a font:
 
-  Disabling maps:
-    updmap.cfg files with higher priority (listed earlier) can disable
-    maps mentioned in lower priority (listed later) updmap.cfg files by
-    writing, e.g.,
-      \#! Map mapname.map
-    or
-      \#! MixedMap mapname.map
-    in the higher-priority updmap.cfg file. 
+  If a font is defined in more than one map file, then the definition
+  coming from the first-listed updmap.cfg is used.  If a font is
+  defined multiple times within the same map file, one is chosen
+  arbitrarily.  In both cases a warning is issued.
 
-    As an example, suppose you have a copy of MathTime Pro fonts
-    and want to disable the Belleek version of the fonts; that is,
-    disable the map belleek.map.  You can create the file
-    \$TEXMFCONFIG/web2c/updmap.cfg with the content
-      #! Map belleek.map
-      Map mt-plus.map
-      Map mt-yy.map
-    and call $prg.
+Disabling maps:
 
-  updmap writes the map files for dvips (psfonts.map) and pdftex
-  (pdftex.map) to the TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/
-  directories.   
+  updmap.cfg files with higher priority (listed earlier) can disable
+  maps mentioned in lower priority (listed later) updmap.cfg files by
+  writing, e.g.,
+    \#! Map mapname.map
+  or
+    \#! MixedMap mapname.map
+  in the higher-priority updmap.cfg file. 
 
-  The log file is written to TEXMFVAR/web2c/updmap.log.
+  As an example, suppose you have a copy of MathTime Pro fonts
+  and want to disable the Belleek version of the fonts; that is,
+  disable the map belleek.map.  You can create the file
+  \$TEXMFCONFIG/web2c/updmap.cfg with the content
+    #! Map belleek.map
+    Map mt-plus.map
+    Map mt-yy.map
+  and call $prg.
 
-  When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
-  instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the only
-  difference between updmap-sys and updmap.
+updmap writes the map files for dvips (psfonts.map) and pdftex
+(pdftex.map) to the TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/
+directories.   
 
-  Other locations may be used if you give them on the command line, or
-  these trees don't exist, or you are not using the original TeX Live.
+The log file is written to TEXMFVAR/web2c/updmap.log.
 
-  To see the precise locations of the various files that
-  will be read and written, give the -n option (or read the source).
+When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
+instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the only
+difference between updmap-sys and updmap.
+
+Other locations may be used if you give them on the command line, or
+these trees don't exist, or you are not using the original TeX Live.
+
+To see the precise locations of the various files that
+will be read and written, give the -n option (or read the source).
 
 For step-by-step instructions on making new fonts known to TeX, read
 http://tug.org/fonts/fontinstall.html.  For even more terse
